@@ -10,7 +10,7 @@ options = r"\[.*?\]"
 spaces = r"[ \t]*"
 
 get_pattern_brace = lambda index: rf"\{{((?:[^{{}}]++|(?{index}))++)\}}"
-get_pattern_env = lambda name: rf"\\begin{spaces}\{{({name})\}}{spaces}({options})?(.*?)\\end{spaces}\{{\1\}}".replace('options', options)
+get_pattern_env = lambda name: rf"\\begin{spaces}\{{({name})\}}{spaces}({options})?(.*?)\\end{spaces}\{{\1\}}"
 get_pattern_command_full = lambda name: rf'\\({name}){spaces}({options})?{spaces}({get_pattern_brace(3)})'
 match_command_name = r'[a-zA-Z]+\*?'
 
@@ -21,7 +21,7 @@ pattern_brace = get_pattern_brace(0)  # {xxx}, group 1: content
 
 pattern_theorem = r"\\newtheorem[ \t]*\{(.+?)\}"  # \newtheorem{xxx}, group 1: name
 pattern_accent = r"\\([`'\"^~=.])(?:\{([a-zA-Z])\}|([a-zA-Z]))"  # match special characters with accents, group 1: accent, group 2/3: normal character
-match_code_accent = rf'{math_code}([A-Z]{{2}})([a-zA-Z])'  # group 1: accent name, group 2: normal character
+match_code_accent = rf'{math_code}([A-Z]{{2}})([a-zA-Z])'  # group 1: accent name, group 2: normal character, e.g. \"o or \"{o}
 list_special = ['\\', '%', '&', '#', '$', '{', '}', ' ']  # all special characters in form of \x
 
 special_character_forward = {
@@ -79,7 +79,7 @@ def modify_after(text):
     return text
 
 
-def replace_latex_objects(text):
+def replace_latex_objects(text, brace=True):
     r"""
     Replaces all LaTeX objects in a given text with the format "{math_code}_{digit1}_{digit2}_..._{digit_last}",
     applies a given function to the resulting text (excluding the "{math_code}_{digit1}_{digit2}_..._{digit_last}" parts),
@@ -89,6 +89,7 @@ def replace_latex_objects(text):
     Returns the processed text and a list of replaced LaTeX objects.
     """
 
+    # You need to make sure that the input does not contain {math_code}
     # define regular expressions for each LaTeX object
     latex_obj_regex = [
         r"\$\$(.*?)\$\$",  # $$ $$
@@ -98,8 +99,9 @@ def replace_latex_objects(text):
         pattern_env,  # \begin{xxx} \end{xxx}
         pattern_command_full,  # \xxx[xxx]{xxx}
         pattern_command_simple,  # \xxx
-        pattern_brace,  # {xxx}
     ]
+    if brace:
+        latex_obj_regex.append(pattern_brace)
 
     # iterate through each LaTeX object and replace with "{math_code}_{digit1}_{digit2}_..._{digit_last}"
     count = 0
@@ -116,7 +118,8 @@ def replace_latex_objects(text):
     return text, replaced_objs
 
 
-def recover_latex_objects(text, replaced_objs, final=False):
+def recover_latex_objects(text, replaced_objs, tolerate_error=False):
+    # recover the latex objects from "replace_latex_objects"
     nobjs = len(replaced_objs)
     matched_indices = []
 
@@ -127,11 +130,12 @@ def recover_latex_objects(text, replaced_objs, final=False):
             return replaced_objs[index]
         else:
             if test_environment:
-                assert final
+                assert tolerate_error
             return '???'
 
     text = modify_text(text, modify_after)
     pattern = re.compile(match_code_replace)
+    # count number of mismatch
     total_num = 0
     while True:
         text, num_modify = pattern.subn(lambda match: get_obj(match.group(1)), text)
@@ -174,10 +178,11 @@ def split_latex_document(text, begin_code, end_code):
 
 
 def process_specific_env(latex, function, env_name):
+    # find all patterns of \begin{env_name}[options] content \end{env_name}
+    # then replace `content` by `function(content)`
     pattern = regex.compile(get_pattern_env(env_name), regex.DOTALL)
 
     def process_function(match):
-        # \begin{env_name}[options] content \end{env_name}
         name = match.group(1)
         assert re.match(env_name, name)
         options = match.group(2)
@@ -190,10 +195,11 @@ def process_specific_env(latex, function, env_name):
 
 
 def process_specific_command(latex, function, command_name):
+    # find all patterns of # \{command_name}[options]{content}
+    # then replace `content` by `function(content)`
     pattern = regex.compile(get_pattern_command_full(command_name), regex.DOTALL)
 
     def process_function(match):
-        # \{command_name}[options]{content}
         name = match.group(1)
         assert re.match(command_name, name)
         options = match.group(2)
@@ -205,6 +211,22 @@ def process_specific_command(latex, function, command_name):
     return pattern.sub(process_function, latex)
 
 
+def process_leading_level_brace(latex, function):
+    # leading level means that the {xxx} is not inside other objects, i.e. \command{} or \begin{xxx} \end{xxx}
+    # replace `{ content }` by `{ function(content) }`
+    text, envs = replace_latex_objects(latex, brace=False)
+
+    def process_function(match):
+        content = match.group(1)
+        # function here is translate_paragraph_latex, which cannot contain replaced environments
+        processed_content = function(recover_latex_objects(content, envs)[0])
+        return rf'{{ {processed_content} }}'
+
+    text = regex.compile(pattern_brace, regex.DOTALL).sub(process_function, text)
+    latex = recover_latex_objects(text, envs)[0]
+    return latex
+
+
 def remove_blank_lines(text):
     pattern = re.compile(r'\n\n+')
     text = pattern.sub('\n', text)
@@ -273,6 +295,8 @@ def recover_special(text):
 
 def replace_accent(text):
     def replace_function(match):
+        # if it is \"{o}, then special is ", char1 is o, char2 is None
+        # if it is \"o, then special is ", char1 is None, char2 is o
         special = match.group(1)
         char1 = match.group(2)
         char2 = match.group(3)
@@ -302,6 +326,8 @@ def recover_accent(text):
 
 
 def combine_split_to_sentences(text):
+    # if two lines are separately by only one \n, in latex they are in the same paragraph so we combine them in the same line
+    # However we don't combine them if the second line does not start from normal letters (so usually some latex commands)
     n = len(math_code)
     pattern = re.compile(r'\n(\s*([^\s]+))')
 
